library ieee;
use ieee.std_logic_1164.all;

entity SFR_FSM is
port (clk: in std_logic;
      sfr4: in std_logic_vector (2 downto 0);      -- SFR4 Control Register bit 2 to 0.
      CPUWAITn: in std_logic;                      --input from Core1553
      Resetn: in std_logic;                        --global power on reset
      CPUDOUT: in std_logic_vector(15 downto 0);   -- Read data input from Core1553B
      sfr7: out std_logic_vector(7 downto 0);      -- SFR7 Status Regsiter
      CPUWRn: out std_logic_vector(1 downto 0);    --Output to Core1553
      CPURDn: out std_logic;                       --Output to Core1553
      latch_RD: out std_logic_vector(15 downto 0); --read data output to Core8051
      CPUMEM: out std_logic);                       --Output to Core1553
end SFR_FSM;

architecture FSM of SFR_FSM is

type state_values is (idle, read, write);
signal present_st, next_st: state_values;

signal CPURDn_int: std_logic;
--signal CPUMEM: std_logic;
--signal CPUWRn:std_logic_vector(1 downto 0);
--signal sfr7:std_logic_vector(1 downto 0);
--signal latch_RD:std_logic_vector(1 downto 0);


begin

--FSM state transition
process(clk,Resetn)
begin
if (Resetn = '0') then
  present_st <= idle;
elsif (clk ='1' and clk'event) then
  present_st <= next_st;  
end if;
end process;

--FSM state definition
process (present_st, sfr4, CPUWAITn)
begin
case present_st is
   when idle => case sfr4 is 
                when "000" => next_st<= idle;
                when "011" => next_st<= write;
                when "101" => next_st<= read;
                when others => next_st<= idle;
                end case;
   when write => case CPUWAITn is
                 when '0' => next_st<= write;
                 when '1' => next_st<= idle;
                 end case;
   when read => case CPUWAITn is 
                when '0' => next_st<= read;
                when '1' => next_st<= idle;
                end case;
  
   when others => next_st <= idle;
end case;
end process;

-- FSM Output definition
process (present_st)
begin
case present_st is 
when idle =>  sfr7 <= "00000000";
              CPUWRn <= "11";
              CPURDn_int <= '1';
              CPUMEM <= '0';
when write => sfr7 <= "00000001";
              CPUWRn <= "00";
              CPURDn_int <= '1';
              CPUMEM <= '1';
when read =>  sfr7 <= "00000001";
              CPUWRn <= "11";
              CPURDn_int <= '0';
              CPUMEM <= '1';
when others =>  sfr7 <= "00000000";
                CPUWRn <= "11";
                CPURDn_int <= '1';
                CPUMEM <= '0';
end case;
end process;

-- Latching the read data from Core1553 to Core8051
process(CLK)
begin
if (CLK'event and CLK= '1') then
 if ( CPURDN_int='0') then
    latch_RD <= CPUDOUT;
  end if;
end if;
end process;


CPURDn <= CPURDn_int;

end FSM;
